@@ -5,6 +5,7 @@
 from __future__ import unicode_literals
 
 import json
+import warnings
 from ctypes import addressof, byref, c_double
 
 from django.contrib.gis import gdal
@@ -20,9 +21,12 @@ from django.contrib.gis.geos.prototypes.io import (
     ewkb_w, wkb_r, wkb_w, wkt_r, wkt_w,
 )
 from django.utils import six
+from django.utils.deconstruct import deconstructible
+from django.utils.deprecation import RemovedInDjango20Warning
 from django.utils.encoding import force_bytes, force_text
 
 
+@deconstructible
 class GEOSGeometry(GEOSBase, ListMixin):
     "A class that, generally, encapsulates a GEOS geometry."
 
@@ -121,8 +125,10 @@ class GEOSGeometry(GEOSBase, ListMixin):
         Destroys this Geometry; in other words, frees the memory used by the
         GEOS C++ object.
         """
-        if self._ptr and capi:
+        try:
             capi.destroy_geom(self._ptr)
+        except (AttributeError, TypeError):
+            pass  # Some part might already have been garbage collected
 
     def __copy__(self):
         """
@@ -292,6 +298,14 @@ class GEOSGeometry(GEOSBase, ListMixin):
         "Returns true if other.within(this) returns true."
         return capi.geos_contains(self.ptr, other.ptr)
 
+    def covers(self, other):
+        """
+        Return True if the DE-9IM Intersection Matrix for the two geometries is
+        T*****FF*, *T****FF*, ***T**FF*, or ****T*FF*. If either geometry is
+        empty, return False.
+        """
+        return capi.geos_covers(self.ptr, other.ptr)
+
     def crosses(self, other):
         """
         Returns true if the DE-9IM intersection matrix for the two Geometries
@@ -356,7 +370,8 @@ class GEOSGeometry(GEOSBase, ListMixin):
         return capi.geos_within(self.ptr, other.ptr)
 
     # #### SRID Routines ####
-    def get_srid(self):
+    @property
+    def srid(self):
         "Gets the SRID for the geometry, returns None if no SRID is set."
         s = capi.geos_get_srid(self.ptr)
         if s == 0:
@@ -364,27 +379,38 @@ class GEOSGeometry(GEOSBase, ListMixin):
         else:
             return s
 
-    def set_srid(self, srid):
+    @srid.setter
+    def srid(self, srid):
         "Sets the SRID for the geometry."
-        capi.geos_set_srid(self.ptr, srid)
-    srid = property(get_srid, set_srid)
+        capi.geos_set_srid(self.ptr, 0 if srid is None else srid)
+
+    def get_srid(self):
+        warnings.warn(
+            "`get_srid()` is deprecated, use the `srid` property instead.",
+            RemovedInDjango20Warning, 2
+        )
+        return self.srid
+
+    def set_srid(self, srid):
+        warnings.warn(
+            "`set_srid()` is deprecated, use the `srid` property instead.",
+            RemovedInDjango20Warning, 2
+        )
+        self.srid = srid
 
     # #### Output Routines ####
     @property
     def ewkt(self):
         """
-        Returns the EWKT (SRID + WKT) of the Geometry. Note that Z values
-        are only included in this representation if GEOS >= 3.3.0.
+        Returns the EWKT (SRID + WKT) of the Geometry.
         """
-        if self.get_srid():
-            return 'SRID=%s;%s' % (self.srid, self.wkt)
-        else:
-            return self.wkt
+        srid = self.srid
+        return 'SRID=%s;%s' % (srid, self.wkt) if srid else self.wkt
 
     @property
     def wkt(self):
         "Returns the WKT (Well-Known Text) representation of this Geometry."
-        return wkt_w(3 if self.hasz else 2).write(self).decode()
+        return wkt_w(dim=3 if self.hasz else 2, trim=True).write(self).decode()
 
     @property
     def hex(self):
@@ -395,7 +421,7 @@ class GEOSGeometry(GEOSBase, ListMixin):
         """
         # A possible faster, all-python, implementation:
         #  str(self.wkb).encode('hex')
-        return wkb_w(3 if self.hasz else 2).write_hex(self)
+        return wkb_w(dim=3 if self.hasz else 2).write_hex(self)
 
     @property
     def hexewkb(self):
@@ -404,7 +430,7 @@ class GEOSGeometry(GEOSBase, ListMixin):
         extension of the WKB specification that includes SRID value that are
         a part of this geometry.
         """
-        return ewkb_w(3 if self.hasz else 2).write_hex(self)
+        return ewkb_w(dim=3 if self.hasz else 2).write_hex(self)
 
     @property
     def json(self):
@@ -494,14 +520,18 @@ class GEOSGeometry(GEOSBase, ListMixin):
             else:
                 return
 
-        if (srid is None) or (srid < 0):
-            raise GEOSException("Calling transform() with no SRID set is not supported")
-
         if not gdal.HAS_GDAL:
             raise GEOSException("GDAL library is not available to transform() geometry.")
 
+        if isinstance(ct, gdal.CoordTransform):
+            # We don't care about SRID because CoordTransform presupposes
+            # source SRS.
+            srid = None
+        elif srid is None or srid < 0:
+            raise GEOSException("Calling transform() with no SRID set is not supported")
+
         # Creating an OGR Geometry, which is then transformed.
-        g = self.ogr
+        g = gdal.OGRGeometry(self.wkb, srid)
         g.transform(ct)
         # Getting a new GEOS pointer
         ptr = wkb_r().read(g.wkb)
@@ -603,6 +633,11 @@ class GEOSGeometry(GEOSBase, ListMixin):
         """
         return self._topology(capi.geos_symdifference(self.ptr, other.ptr))
 
+    @property
+    def unary_union(self):
+        "Return the union of all the elements of this geometry."
+        return self._topology(capi.geos_unary_union(self.ptr))
+
     def union(self, other):
         "Returns a Geometry representing all the points in this Geometry and other."
         return self._topology(capi.geos_union(self.ptr, other.ptr))
@@ -652,7 +687,7 @@ class GEOSGeometry(GEOSBase, ListMixin):
         return GEOSGeometry(capi.geom_clone(self.ptr), srid=self.srid)
 
 
-class ProjectInterpolateMixin(object):
+class LinearGeometryMixin(object):
     """
     Used for LineString and MultiLineString.
     """
@@ -673,3 +708,17 @@ class ProjectInterpolateMixin(object):
         if not isinstance(point, Point):
             raise TypeError('locate_point argument must be a Point')
         return capi.geos_project_normalized(self.ptr, point.ptr)
+
+    @property
+    def merged(self):
+        """
+        Return the line merge of this Geometry.
+        """
+        return self._topology(capi.geos_linemerge(self.ptr))
+
+    @property
+    def closed(self):
+        """
+        Return whether or not this Geometry is closed.
+        """
+        return capi.geos_isclosed(self.ptr)
