@@ -1,7 +1,6 @@
 """Default tags used by the template system, available to all templates."""
 from __future__ import unicode_literals
 
-import os
 import re
 import sys
 import warnings
@@ -10,7 +9,6 @@ from itertools import cycle as itertools_cycle, groupby
 
 from django.conf import settings
 from django.utils import six, timezone
-from django.utils.deprecation import RemovedInDjango110Warning
 from django.utils.encoding import force_text, smart_text
 from django.utils.html import conditional_escape, format_html
 from django.utils.lorem_ipsum import paragraphs, words
@@ -19,7 +17,7 @@ from django.utils.safestring import mark_safe
 from .base import (
     BLOCK_TAG_END, BLOCK_TAG_START, COMMENT_TAG_END, COMMENT_TAG_START,
     SINGLE_BRACE_END, SINGLE_BRACE_START, VARIABLE_ATTRIBUTE_SEPARATOR,
-    VARIABLE_TAG_END, VARIABLE_TAG_START, Context, Node, NodeList, Template,
+    VARIABLE_TAG_END, VARIABLE_TAG_START, Context, Node, NodeList,
     TemplateSyntaxError, VariableDoesNotExist, kwarg_re,
     render_value_in_context, token_kwargs,
 )
@@ -191,28 +189,19 @@ class ForNode(Node):
                 if unpack:
                     # If there are multiple loop variables, unpack the item into
                     # them.
-
-                    # To complete this deprecation, remove from here to the
-                    # try/except block as well as the try/except itself,
-                    # leaving `unpacked_vars = ...` and the "else" statements.
-                    if not isinstance(item, (list, tuple)):
-                        len_item = 1
-                    else:
+                    try:
                         len_item = len(item)
+                    except TypeError:  # not an iterable
+                        len_item = 1
                     # Check loop variable count before unpacking
                     if num_loopvars != len_item:
-                        warnings.warn(
+                        raise ValueError(
                             "Need {} values to unpack in for loop; got {}. "
-                            "This will raise an exception in Django 1.10."
                             .format(num_loopvars, len_item),
-                            RemovedInDjango110Warning)
-                    try:
-                        unpacked_vars = dict(zip(self.loopvars, item))
-                    except TypeError:
-                        pass
-                    else:
-                        pop_context = True
-                        context.update(unpacked_vars)
+                        )
+                    unpacked_vars = dict(zip(self.loopvars, item))
+                    pop_context = True
+                    context.update(unpacked_vars)
                 else:
                     context[self.loopvars[0]] = item
 
@@ -373,44 +362,6 @@ class RegroupNode(Node):
         return ''
 
 
-def include_is_allowed(filepath, allowed_include_roots):
-    filepath = os.path.abspath(filepath)
-    for root in allowed_include_roots:
-        if filepath.startswith(root):
-            return True
-    return False
-
-
-class SsiNode(Node):
-    def __init__(self, filepath, parsed):
-        self.filepath = filepath
-        self.parsed = parsed
-
-    def render(self, context):
-        filepath = self.filepath.resolve(context)
-
-        if not include_is_allowed(filepath, context.template.engine.allowed_include_roots):
-            if settings.DEBUG:
-                return "[Didn't have permission to include file]"
-            else:
-                return ''  # Fail silently for invalid includes.
-        try:
-            with open(filepath, 'r') as fp:
-                output = fp.read()
-        except IOError:
-            output = ''
-        if self.parsed:
-            try:
-                t = Template(output, name=filepath, engine=context.template.engine)
-                return t.render(context)
-            except TemplateSyntaxError as e:
-                if settings.DEBUG:
-                    return "[Included template had syntax error: %s]" % e
-                else:
-                    return ''  # Fail silently for invalid included templates.
-        return output
-
-
 class LoadNode(Node):
     def render(self, context):
         return ''
@@ -467,53 +418,28 @@ class URLNode(Node):
         self.asvar = asvar
 
     def render(self, context):
-        from django.core.urlresolvers import reverse, NoReverseMatch
+        from django.urls import reverse, NoReverseMatch
         args = [arg.resolve(context) for arg in self.args]
         kwargs = {
             smart_text(k, 'ascii'): v.resolve(context)
             for k, v in self.kwargs.items()
         }
-
         view_name = self.view_name.resolve(context)
-
         try:
             current_app = context.request.current_app
         except AttributeError:
-            # Leave only the else block when the deprecation path for
-            # Context.current_app completes in Django 1.10.
-            # Can also remove the Context.is_current_app_set property.
-            if context.is_current_app_set:
-                current_app = context.current_app
-            else:
-                try:
-                    current_app = context.request.resolver_match.namespace
-                except AttributeError:
-                    current_app = None
-
-        # Try to look up the URL twice: once given the view name, and again
-        # relative to what we guess is the "main" app. If they both fail,
-        # re-raise the NoReverseMatch unless we're using the
-        # {% url ... as var %} construct in which case return nothing.
+            try:
+                current_app = context.request.resolver_match.namespace
+            except AttributeError:
+                current_app = None
+        # Try to look up the URL. If it fails, raise NoReverseMatch unless the
+        # {% url ... as var %} construct is used, in which case return nothing.
         url = ''
         try:
             url = reverse(view_name, args=args, kwargs=kwargs, current_app=current_app)
         except NoReverseMatch:
-            exc_info = sys.exc_info()
-            if settings.SETTINGS_MODULE:
-                project_name = settings.SETTINGS_MODULE.split('.')[0]
-                try:
-                    url = reverse(project_name + '.' + view_name,
-                              args=args, kwargs=kwargs,
-                              current_app=current_app)
-                except NoReverseMatch:
-                    if self.asvar is None:
-                        # Re-raise the original exception, not the one with
-                        # the path relative to the project. This makes a
-                        # better error message.
-                        six.reraise(*exc_info)
-            else:
-                if self.asvar is None:
-                    raise
+            if self.asvar is None:
+                raise
 
         if self.asvar:
             context[self.asvar] = url
@@ -658,15 +584,6 @@ def cycle(parser, token):
     if len(args) < 2:
         raise TemplateSyntaxError("'cycle' tag requires at least two arguments")
 
-    if ',' in args[1]:
-        warnings.warn(
-            "The old {% cycle %} syntax with comma-separated arguments is deprecated.",
-            RemovedInDjango110Warning,
-        )
-        # Backwards compatibility: {% cycle a,b %} or {% cycle a,b as foo %}
-        # case.
-        args[1:2] = ['"%s"' % arg for arg in args[1].split(",")]
-
     if len(args) == 2:
         # {% cycle foo %} case.
         name = args[1]
@@ -756,7 +673,7 @@ def do_filter(parser, token):
 @register.tag
 def firstof(parser, token):
     """
-    Outputs the first variable passed that is not False, without escaping.
+    Outputs the first variable passed that is not False.
 
     Outputs nothing if all the passed variables are False.
 
@@ -767,11 +684,11 @@ def firstof(parser, token):
     This is equivalent to::
 
         {% if var1 %}
-            {{ var1|safe }}
+            {{ var1 }}
         {% elif var2 %}
-            {{ var2|safe }}
+            {{ var2 }}
         {% elif var3 %}
-            {{ var3|safe }}
+            {{ var3 }}
         {% endif %}
 
     but obviously much cleaner!
@@ -1091,42 +1008,6 @@ def ifchanged(parser, token):
     return IfChangedNode(nodelist_true, nodelist_false, *values)
 
 
-@register.tag
-def ssi(parser, token):
-    """
-    Outputs the contents of a given file into the page.
-
-    Like a simple "include" tag, the ``ssi`` tag includes the contents
-    of another file -- which must be specified using an absolute path --
-    in the current page::
-
-        {% ssi "/home/html/ljworld.com/includes/right_generic.html" %}
-
-    If the optional "parsed" parameter is given, the contents of the included
-    file are evaluated as template code, with the current context::
-
-        {% ssi "/home/html/ljworld.com/includes/right_generic.html" parsed %}
-    """
-    warnings.warn(
-        "The {% ssi %} tag is deprecated. Use the {% include %} tag instead.",
-        RemovedInDjango110Warning,
-    )
-
-    bits = token.split_contents()
-    parsed = False
-    if len(bits) not in (2, 3):
-        raise TemplateSyntaxError("'ssi' tag takes one argument: the path to"
-                                  " the file to be included")
-    if len(bits) == 3:
-        if bits[2] == 'parsed':
-            parsed = True
-        else:
-            raise TemplateSyntaxError("Second (optional) argument to %s tag"
-                                      " must be 'parsed'" % bits[0])
-    filepath = parser.compile_filter(bits[1])
-    return SsiNode(filepath, parsed)
-
-
 def find_library(parser, name):
     try:
         return parser.libraries[name]
@@ -1398,61 +1279,40 @@ def templatetag(parser, token):
 @register.tag
 def url(parser, token):
     """
-    Returns an absolute URL matching given view with its parameters.
+    Return an absolute URL matching the given view with its parameters.
 
     This is a way to define links that aren't tied to a particular URL
     configuration::
 
-        {% url "path.to.some_view" arg1 arg2 %}
+        {% url "url_name" arg1 arg2 %}
 
         or
 
-        {% url "path.to.some_view" name1=value1 name2=value2 %}
+        {% url "url_name" name1=value1 name2=value2 %}
 
-    The first argument is a path to a view. It can be an absolute Python path
-    or just ``app_name.view_name`` without the project name if the view is
-    located inside the project.
+    The first argument is a django.conf.urls.url() name. Other arguments are
+    space-separated values that will be filled in place of positional and
+    keyword arguments in the URL. Don't mix positional and keyword arguments.
+    All arguments for the URL must be present.
 
-    Other arguments are space-separated values that will be filled in place of
-    positional and keyword arguments in the URL. Don't mix positional and
-    keyword arguments.
+    For example, if you have a view ``app_name.views.client_details`` taking
+    the client's id and the corresponding line in a URLconf looks like this::
 
-    All arguments for the URL should be present.
-
-    For example if you have a view ``app_name.client`` taking client's id and
-    the corresponding line in a URLconf looks like this::
-
-        ('^client/(\d+)/$', 'app_name.client')
+        url('^client/(\d+)/$', views.client_details, name='client-detail-view')
 
     and this app's URLconf is included into the project's URLconf under some
     path::
 
-        ('^clients/', include('project_name.app_name.urls'))
+        url('^clients/', include('app_name.urls'))
 
     then in a template you can create a link for a certain client like this::
 
-        {% url "app_name.client" client.id %}
+        {% url "client-detail-view" client.id %}
 
     The URL will look like ``/clients/client/123/``.
 
-    The first argument can also be a named URL instead of the Python path to
-    the view callable. For example if the URLconf entry looks like this::
-
-        url('^client/(\d+)/$', name='client-detail-view')
-
-    then in the template you can use::
-
-        {% url "client-detail-view" client.id %}
-
-    There is even another possible value type for the first argument. It can be
-    the name of a template variable that will be evaluated to obtain the view
-    name or the URL name, e.g.::
-
-        {% with view_path="app_name.client" %}
-        {% url view_path client.id %}
-        {% endwith %}
-
-        or,
+    The first argument may also be the name of a template variable that will be
+    evaluated to obtain the view name or the URL name, e.g.::
 
         {% with url_name="client-detail-view" %}
         {% url url_name client.id %}
@@ -1460,8 +1320,7 @@ def url(parser, token):
     """
     bits = token.split_contents()
     if len(bits) < 2:
-        raise TemplateSyntaxError("'%s' takes at least one argument"
-                                  " (path to a view)" % bits[0])
+        raise TemplateSyntaxError("'%s' takes at least one argument, the name of a url()." % bits[0])
     viewname = parser.compile_filter(bits[1])
     args = []
     kwargs = {}
